@@ -244,7 +244,7 @@ int merge_boxes( struct box *box1, struct box *box2 ) {
     tmpbox.frame_per[ tmpbox.num_frames ] = bsmaller->frame_per[ i3 ];
     tmpbox.num_frames++;
     if (tmpbox.num_frames>=MaxNumFrames) {
-      if (JOB->cfg.verbose)
+      if (OCR_JOB->cfg.verbose)
         fprintf(stderr,"\nDBG merge_boxes MaxNumFrames reached");
       break;
     }
@@ -260,7 +260,7 @@ int merge_boxes( struct box *box1, struct box *box2 ) {
   memcpy(box1->frame_vector,
          tmpbox.frame_vector,sizeof(int)*2*MaxFrameVectors);
 #if 0
-  if ((JOB->cfg.verbose&48)==48) {
+  if ((OCR_JOB->cfg.verbose&48)==48) {
     fprintf(stderr,"\nDBG merge_boxes_result:"); out_x(box1); }
 #endif
   return 0;
@@ -277,37 +277,145 @@ int merge_boxes( struct box *box1, struct box *box2 ) {
  *        depending on direction (rotation) of outside connections
  *  - 3th shift outside vectors to crossing points
  *  - split add this points, connect only in-out...out-in,
- *  - cutting can result in more objects
+ *  - cutting can result in more (fi) or less objects (ke)
  * ToDo:
- *       dont connect  --1---2--------3----4--  new-y1 (inside above not drawn)
- *                        \   \->>>>-/    /             outside
- *                         \----<<<<-----/      old-y1
+ *       dont connect  --1---4--------3----2--  new-y1 (inside above not drawn)
+ *                        \   \-<<<<-/    /             outside
+ *                         \---->>>>-----/      old-y1
  *                            |======| subtractable? 
  *
  *       only connect  --1---2--------3----4--  new-y1
  *                        \>>/        \>>>/     old-y1  outside
- *  ToDo: what about cutting 2 frames (example: 2fold melted MN)
- *        better restart framing algo?
+ *  ToDo: what about cutting 2 frames (example: 2fold melted MN,ke)
+ *        better restart framing algo (modified pgm2asc.frame_vector())?
+ *        (would be require pixel last 3 bits modifications?)
  *
  *  ToDo: new vol, per
+ * sample: tmp08/gocr0810* 2010-10-12
+ frame 0 ( +66, 50,21)
+  #00  0  0 #01  0  8 #02  0  6 #03  1  5 #04  4  8 #05  5  8 #06  6  7
+  #07  7  8 #08 11  8 #09  8  8 #10  6  6 #11  7  5 #12 12  5 #13 12  3
+  #14 11  2 #15  7  2 #16  6  3 #17  5  2 #18  4  2 #19  1  5 #20  0  4
+ frame 1 ( -10, 13, 8)  
+  #21  4  3 #22  5  2 #23  6  3 #24  6  7 #25  5  8 #26  2  5 #27  2  4 
+  #28  3  3
+ frame 2 (  -8, 12, 7)  
+  #29  7  3 #30  8  2 #31 10  2 #32 12  4 #33 11  5 #34  7  5 #35  6  4
+  # list pattern  d=  13   9
+  $............  @............<-
+  @............  @............ 
+  @...$$.$$@$$.  @...@@.@@@@@.<
+  @..$$.$$...@$  @..@@.@@...@@
+  $.$...$.....$  @.@...@.....@
+  @$$...@$@@@$$  @@@...@@@@@@@  
+  $.@@..$......  @.@@..@......  
+  @..@@.$@.....  @..@@.@@.....  
+  $...$$.$$@@$.  @...@@.@@@@@.<-
+       ^cut           ^cut     
+  1st) get a list of cutting positions and sort them (going along border):
+     [#05-#06(5,8,io)   #16-#17(5,2,oi)] [#22-#23(5,2,io)   #24-#25(5,8,oi)]
+     [#05-#06(5,8,io)  [#24-#25(5,8,oi)   #22-#23(5,2,io)]  #16-#17(5,2,oi)]
+     // positive rotation momentum io(0)->io(1)->oi(0)->oi(1)->back
+     // io=inside_to_outside oi=outside_to_inside
+  2nd) 
+   rebuild frames ... #04 #05 - #25 #26 #27 #28 #21 #22 #23 - #17 #18 ...
  */
+#if 0  // new version 1010-10 works on cutting multiple frames
+#define MAXNUMCUTS 8   // used in function cut_box() only
 int cut_box( struct box *box1) {
-  int i1, i2, i3, i4, x, y, lx, ly, dbg=0;
-  if (JOB->cfg.verbose) dbg=1;  // debug level, enlarge to get more output
+  int i1, i2, i3, i4, i4l, i5, x, y, lx, ly, dbg=0;
+  int cut_num = 0,         // number of cuts
+      cut_idx[MAXNUMCUTS], // index to cutted vector
+      cut_dir[MAXNUMCUTS]; // direction of vector inside_outside=1
+                           //                     outside_inside=2
+  struct box tmpbox; // = (*box1); temp. buffer, after some corrections
+#ifdef DO_DEBUG
+  if (OCR_JOB->cfg.verbose) dbg=1+2;  // debug level, enlarge to get more output
+#endif  
   if (dbg) fprintf(stderr,"\n cut box x= %3d %3d", box1->x0, box1->y0);
   /* check if complete frames are outside the box */
   for (i1=0; i1<box1->num_frames; i1++){
-    if (dbg>2) fprintf(stderr,"\n checking frame %d outside", i1);
+    if (dbg>2) fprintf(stderr,"\n checking frame %d ", i1);
+    /* using frame_per as marker first (recalculate later) */
+    i2 = ((i1)?box1->num_frame_vectors[ i1-1 ]:0); // start of this frame
+    i3 =       box1->num_frame_vectors[ i1   ];    // start of next frame
+    x = box1->frame_vector[ i3-1 ][0]; /* initial */
+    y = box1->frame_vector[ i3-1 ][1]; /* initial */
+    i4l = i3 - 1; // initial index of last vector
+    if (x<box1->x0 || x>box1->x1 || y<box1->y0 || y>box1->y1) i5=1; // outside
+    for (i4=i2; i4 < i3; i4++) {
+      lx = x; /* last x = vector[i4l] */
+      ly = y; /* last y = vector[i4l] */
+      x = box1->frame_vector[i4][0];
+      y = box1->frame_vector[i4][1];
+      // fprintf(stderr,"DBG LEV3 i4= %3d  xy= %3d %3d",i4,x,y);
+      /* check if outside */
+      if (x<box1->x0 || x>box1->x1 || y<box1->y0 || y>box1->y1) { // outside
+        /* replace by nearest point at border, ToDo: better crossingpoint */
+        if (i5==0) {  /* inside_to_outside, wrong if it starts outside */
+          if (x < box1->x0) x = box1->frame_vector[i4][0] = box1->x0;
+          if (x > box1->x1) x = box1->frame_vector[i4][0] = box1->x1;
+          if (y < box1->y0) y = box1->frame_vector[i4][1] = box1->y0;
+          if (y > box1->y1) y = box1->frame_vector[i4][1] = box1->y1;
+          if (cut_num<MAXCUTNUM) {
+            cut_idx[cut_num] = i4;
+            cut_dir[cut_num] = 1; // inside_to_outside
+            cut_num++;
+          }
+          /* search other frames for near outside_to_inside and copy frame in */
+        }
+        i5 = 1; // flag: we_are_outside_of_box
+      } else if (i5==1) {   /* outside_to_inside */
+        /* ToDo: better crossing point last vector and border */
+        if (lx < box1->x0) lx = box1->x0;
+        if (lx > box1->x1) lx = box1->x1;
+        if (ly < box1->y0) ly = box1->y0;
+        if (ly > box1->y1) ly = box1->y1;
+        box1->frame_vector[i4l][0] = lx;
+        box1->frame_vector[i4l][1] = ly;
+        if (cut_num<MAXCUTNUM) {
+          cut_idx[cut_num] = i4l;
+          cut_dir[cut_num] = 2; // outside_to_inside
+          cut_num++;
+        }
+        i5 = 0; // flag: we_are_inside_of_box
+      }
+      i4l=i4; // index of last vector
+      // fprintf(stderr," xy= %3d %3d\n",x,y);
+    }
+  }
+  for (i1=0; i1<cut_num; i1+=2) {
+  }
+  if (dbg>2) { fprintf(stderr,"\nDBG cut_box_result:"); out_x(box1); }
+  return 0;
+}
+#else  // old bad version works for one cutted frame only
+int cut_box( struct box *box1) {
+  int i1, i2, i3, i4, x, y, lx, ly, dbg=0;
+#ifdef DO_DEBUG
+  if (OCR_JOB->cfg.verbose) dbg=1+2;  // debug level, enlarge to get more output
+#endif  
+  if (dbg) fprintf(stderr,"\n cut box x= %3d %3d", box1->x0, box1->y0);
+  /* check if complete frames are outside the box */
+  for (i1=0; i1<box1->num_frames; i1++){
+    if (dbg>2) fprintf(stderr,"\n checking frame %d ", i1);
+    /* using frame_per as marker first (recalculate later) */
+    box1->frame_per[i1]=0;
     i2 = ((i1)?box1->num_frame_vectors[ i1-1 ]:0); // this frame
     i3 =       box1->num_frame_vectors[ i1   ];    // next frame
     for (i4=i2; i4 < i3; i4++) {
       x = box1->frame_vector[i4][0];
       y = box1->frame_vector[i4][1];
-      /* break, if one vector is lying inside */
-      if (x>=box1->x0 && x<=box1->x1 && y>=box1->y0 && y<=box1->y1) break;
+      /* 1 if one vector is lying inside */
+      /* 2 if one vector is lying outside */
+      if (x>=box1->x0 && x<=box1->x1
+       && y>=box1->y0 && y<=box1->y1) box1->frame_per[i1]|=1;
+      if (x< box1->x0 || x> box1->x1
+       || y< box1->y0 || y> box1->y1) box1->frame_per[i1]|=2;
     }
-    if (i4==i3) { /* all vectors outside */
-      if (dbg>1) fprintf(stderr,"\n remove frame %d",i1);
+    if (dbg>2) fprintf(stderr,"in1+out2= %d", box1->frame_per[i1]);
+    if (box1->frame_per[i1]==2) { /* all vectors outside */
+      if (dbg>1) fprintf(stderr," remove frame %d",i1);
       /* replace all frames i1,i1+1,... by i1+1,i1+2,... */
       /* replace (x,y) pairs first */
       for (i4=i2; i4<box1->num_frame_vectors[ box1->num_frames-1 ]-(i3-i2);
@@ -316,18 +424,25 @@ int cut_box( struct box *box1) {
         box1->frame_vector[i4][1] = box1->frame_vector[i4+i3-i2][1];
       }
       /* replace the num_frame_vectors */
-      for (i4=i1; i4<box1->num_frames-1; i4++)
+      for (i4=i1; i4<box1->num_frames-1; i4++) {
         box1->num_frame_vectors[ i4 ] =
           box1->num_frame_vectors[ i4+1 ]-(i3-i2);
+        box1->frame_vol[i4]=box1->frame_vol[i4+1];
+        box1->frame_per[i4]=box1->frame_per[i4+1];
+      }
       box1->num_frames--; i1--;
     }
   }
-  /* remove vectors outside the box */
-  i3=0;
+  /* remove vectors outside the box and connect vectors pointing outside
+   *  to next vectors pointing inside (every frame), see above sample "ke" 
+   *    ToDo: partly implemented, start i2source=search = i2dest=trace,
+  *       i2source>i2dest on reconnect (evl. frame number changed)
+   */
+  i3=0; /* flag 0=inside=no_change 1++=outside_remove_or_concat */
   for (i1=0; i1<box1->num_frames; i1++){
     if (dbg>2) fprintf(stderr,"\n check cutting vectors on frame %d", i1);
-    x = box1->frame_vector[0][0]; /* last x */
-    y = box1->frame_vector[0][1]; /* last y */
+    x = box1->frame_vector[0][0]; /* initial x */
+    y = box1->frame_vector[0][1]; /* initial y */
     /* ToDo: start inside to get a closed object */
     if (x<box1->x0 || x>box1->x1 || y<box1->y0 || y>box1->y1) i3=1;
     for (i2=0; i2<box1->num_frame_vectors[ i1 ]; i2++) {
@@ -339,12 +454,13 @@ int cut_box( struct box *box1) {
       /* check if outside */
       if (x<box1->x0 || x>box1->x1 || y<box1->y0 || y>box1->y1) {
         /* replace by nearest point at border, ToDo: better crossingpoint */
-        if (i3==0) {  /* wrong if it starts outside */
+        if (i3==0) {  /* inside_to_outside, wrong if it starts outside */
           if (x < box1->x0) x = box1->frame_vector[i2][0] = box1->x0;
           if (x > box1->x1) x = box1->frame_vector[i2][0] = box1->x1;
           if (y < box1->y0) y = box1->frame_vector[i2][1] = box1->y0;
           if (y > box1->y1) y = box1->frame_vector[i2][1] = box1->y1;
-        } else {
+          /* search other frames for near outside_to_inside and copy frame in */
+        } else { /* outside_to_outside */
           /* remove vector */
           if (dbg>1) fprintf(stderr,"\n remove vector[%d][%d] x= %2d %2d",i1,i2,x-box1->x0,y-box1->y0);
           for (i4=i2;i4<box1->num_frame_vectors[ box1->num_frames-1 ]-1;i4++) {
@@ -359,15 +475,17 @@ int cut_box( struct box *box1) {
         // fprintf(stderr," outside i3= %d\n",i3);
         continue;
       }
+      /* outside_or_inside to inside */
       // fprintf(stderr," inside  i3= %d",i3);
-      if (i3) { /* ToDo: better crossing point last vector and border */
-          if (lx < box1->x0) lx = box1->x0;
-          if (lx > box1->x1) lx = box1->x1;
-          if (ly < box1->y0) ly = box1->y0;
-          if (ly > box1->y1) ly = box1->y1;
-          x = box1->frame_vector[i2][0] = lx;
-          y = box1->frame_vector[i2][1] = ly;
-          i3 = 0;
+      if (i3) { /* outside_to_inside */
+        /* ToDo: better crossing point last vector and border */
+        if (lx < box1->x0) lx = box1->x0;
+        if (lx > box1->x1) lx = box1->x1;
+        if (ly < box1->y0) ly = box1->y0;
+        if (ly > box1->y1) ly = box1->y1;
+        x = box1->frame_vector[i2][0] = lx;
+        y = box1->frame_vector[i2][1] = ly;
+        i3 = 0;
       }
       // fprintf(stderr," xy= %3d %3d\n",x,y);
     }
@@ -375,4 +493,4 @@ int cut_box( struct box *box1) {
   if (dbg>2) { fprintf(stderr,"\nDBG cut_box_result:"); out_x(box1); }
   return 0;
 }
-
+#endif
