@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000-2010 Joerg Schulenburg
+Copyright (C) GPLv2 2000-2013 Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -32,7 +32,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 // load boxes from database into boxlist (for faster access)
 //   used as alternate engine, comparing chars with database
-int load_db(void) {
+// uses readpnm() and would conflict with multi images
+int load_db(job_t *job) {  // called by gocr.c main()
   FILE *f1;
   char s1[Blen+1],
        s2[Blen+1] = "./db/", /* ToDo: replace by constant! by configure */
@@ -41,10 +42,10 @@ int load_db(void) {
   struct box *box1;
   pix *pp;
 
-  if( JOB->cfg.db_path ) strncpy(s2,JOB->cfg.db_path,Blen-1);
+  if( job->cfg.db_path ) strncpy(s2, job->cfg.db_path, Blen-1);
   i2=strlen(s2);
-  if (JOB->cfg.verbose)
-    fprintf(stderr, "# load database %s %s ... ", s2, JOB->cfg.db_path);
+  if (job->cfg.verbose)
+    fprintf(stderr, "# load database %s %s ... ", s2, job->cfg.db_path);
 
   strncpy(s2+i2,"db.lst",Blen-i2);s2[Blen]=0;
   f1 = fopen(s2, "r");
@@ -73,8 +74,8 @@ int load_db(void) {
     pp = (pix *)malloc(sizeof(pix));
     if( !pp ) fprintf(stderr,"malloc error in load_db pix\n");
 
-    // if (JOB->cfg.verbose) fprintf(stderr,"\n# readpgm %s ",s2);
-    if (readpgm(s2, pp, 0 * JOB->cfg.verbose)!=0) {
+    // if (job->cfg.verbose) fprintf(stderr,"\n# readpgm %s ",s2);
+    if (readpgm(s2, pp, 0 * job->cfg.verbose)!=0) {
       fprintf(stderr,"\ndatabase error: readpgm %s\n", s2);
       exit(-1);
     }
@@ -122,27 +123,27 @@ int load_db(void) {
     box1->m3 = 0;
     box1->m4 = 0;
     box1->p = pp;
-    list_app(&JOB->tmp.dblist, box1);	// append to list
+    list_app(&job->tmp.dblist, box1);	// append to list
 #if 0
      out_x(box1);
 #endif
   }
   fclose(f1);
-  if (JOB->cfg.verbose)
+  if (job->cfg.verbose)
     fprintf(stderr, " %d chars loaded\n", ii);
   return 0;
 }
 
 // expand database from box/boxlist name=db_$utime.pbm
 // this is added in version v0.3.3
-int store_db(struct box *box1) {
+int store_db(struct box *box1, job_t *job) {
   FILE *f1;
   char s2[Blen+1] = "./db/", s3[Blen+1];
   int i2, dx, dy;
   unsigned c_out;
   pix b;	/* temporary mini page */
 
-  if( JOB->cfg.db_path ) strncpy(s2,JOB->cfg.db_path,Blen-1);
+  if( job->cfg.db_path ) strncpy(s2,job->cfg.db_path,Blen-1);
   i2=strlen(s2);
   
   /* add (first) char and time to the file name for better debugging */
@@ -168,7 +169,7 @@ int store_db(struct box *box1) {
   /* store image and infos about the char */
   /* ToDo: store the vector list instead of the pixelarray */
   
-  if (JOB->cfg.verbose)
+  if (job->cfg.verbose)
     fprintf(stderr, "store_db: add file %s to database (nac=%d c=%04x)"
       "\n#",s3, box1->num_ac, c_out);
 
@@ -177,10 +178,13 @@ int store_db(struct box *box1) {
   b.p = (unsigned char *) malloc( dx * dy );
   if( !b.p ){
     fprintf( stderr, "\nFATAL: malloc failed, skip store_db" );
+    fclose(f1);
     return 2;
   }
-  if (copybox(box1->p, box1->x0, box1->y0, dx, dy, &b, dx * dy))
+  if (copybox(box1->p, box1->x0, box1->y0, dx, dy, &b, dx * dy)) {
+    fclose(f1);
     return -1;
+  }
                           
   writepbm(s2,&b); /* What is to do on error? */
   free(b.p);
@@ -210,10 +214,10 @@ int store_db(struct box *box1) {
    it prints out a part of pixmap b at point x0,y0 to stderr
    using dots .,; if no pixel, and @xoO for pixels
  */
-void out_env(struct box *px ){
+void out_env(struct box *px, job_t *job){
   int x0,y0,x1,y1,dx,dy,x,y,x2,y2,yy0,tx,ty,i,cs;
   char c1, c2; pix *b;
-  cs=JOB->cfg.cs;
+  cs=job->cfg.cs;
   yy0=px->y0;
   { /* overwrite rest of arguments */
     b=px->p;
@@ -289,11 +293,11 @@ void out_env(struct box *px ){
 */
 // #define DEBUG 2
 int distance2( pix *p1, struct box *box1,
-               pix *p2, struct box *box2, int cs){
+               pix *p2, struct box *box2, int cs, int vvv){
    int rc=0,x,y,v1,v2,i1,i2,rgood=0,rbad=0,
        x1,y1,x2,y2,dx,dy,dx1,dy1,dx2,dy2,tx,ty;
 #if DEBUG == 2
-  if(JOB->cfg.verbose)
+  if (vvv)
     fprintf(stderr," DEBUG: distance2\n");
 #endif
    x1=box1->x0;y1=box1->y0;x2=box2->x0;y2=box2->y0;
@@ -329,7 +333,7 @@ int distance2( pix *p1, struct box *box1,
    /* if width/high is not correct add badness */
    rc += ( abs(dx1*dy2-dx2*dy1) * 10 ) / (dy1*dy2);
    if (rc>100) rc=100;
-   if(/* rc<10 && */ JOB->cfg.verbose /* &1024 */){
+   if(/* rc<10 && */ vvv /* &1024 */){
 #if DEBUG == 2
      fprintf(stderr," distance2 rc=%d rgood=%d rbad=%d\n",rc,rgood,rbad);
 //     out_b(NULL,p1,box1->x0,box1->y0,box1->x1-box1->x0+1,
@@ -343,26 +347,27 @@ int distance2( pix *p1, struct box *box1,
    return rc;
 }
 
-wchar_t ocr_db(struct box *box1) {
+wchar_t ocr_db(struct box *box1, job_t *job) {
   int dd = 1000, dist = 1000;
   wchar_t c = UNKNOWN;
   unsigned char buf[200]; /* Oct08 JS: add unsigned to avoid UTF problems */
   Box *box2, *box3;
   
-  if (!list_empty(&JOB->tmp.dblist)){ 
-    box3 = (Box *)list_get_header(&JOB->tmp.dblist);
-    if(JOB->cfg.verbose)
+  if (!list_empty(&job->tmp.dblist)){ 
+    box3 = (Box *)list_get_header(&job->tmp.dblist);
+    if(job->cfg.verbose)
       fprintf(stderr,"\n#DEBUG: ocr_db (%d,%d) ",box1->x0, box1->y0);
 
-    for_each_data(&JOB->tmp.dblist) {
-      box2 = (Box *)list_get_current(&JOB->tmp.dblist);
+    for_each_data(&job->tmp.dblist) {
+      box2 = (Box *)list_get_current(&job->tmp.dblist);
       /* do preselect!!! distance() slowly */
-      dd = distance2( box2->p, box2, box1->p, box1, JOB->cfg.cs);
+      dd = distance2( box2->p, box2, box1->p, box1,
+                      job->cfg.cs, job->cfg.verbose);
       if (dd <= dist) {  /* new best fit */ 
         dist = dd;
         box3 = box2; /* box3 is a pointer and not copied box2 */
 
-        if (dist<100 && 100-dist >= JOB->cfg.certainty) {
+        if (dist<100 && 100-dist >= job->cfg.certainty) {
           /* some deviation of the pattern is tolerated */
           int i, wa;
           for (i=0;i<box3->num_ac;i++) {
@@ -371,7 +376,7 @@ wchar_t ocr_db(struct box *box1) {
            else              setac(box1,box3->tac[i],wa);
           }
           if (box3->num_ac) c=box3->tac[0]; /* 0 for strings (!UNKNOWN) */
-          if (JOB->cfg.verbose)
+          if (job->cfg.verbose)
             fprintf(stderr, " dist=%4d c= %c 0x%02x %s  wc= %3d", dist,
                ((box3->c>32 && box3->c<127) ? (char) box3->c : '.'),
                (int)box3->c, ((box3->tas[0])?box3->tas[0]:""), box3->wac[0]);
@@ -380,15 +385,15 @@ wchar_t ocr_db(struct box *box1) {
                        || !strchr ("l1|I0O", box3->c)))
           break; /* speedup if found */
       }
-    } end_for_each(&JOB->tmp.dblist);
+    } end_for_each(&job->tmp.dblist);
     
   }
 
-  if( (JOB->cfg.mode&128) != 0 && c == UNKNOWN ) { /* prompt the user */
+  if( (job->cfg.mode&128) != 0 && c == UNKNOWN ) { /* prompt the user */
     /* should the output go to stderr or special pipe??? */
     int utf8_ok=0;  /* trigger this flag if input is ok */
     int i, endchar; /* index */
-    out_env(box1);  /* old: out_x(box1); */
+    out_env(box1, job);  /* old: out_x(box1); */
     fprintf(stderr,"The above pattern was not recognized.\n"
       "Enter UTF8 char or string for above pattern. Leave empty if unsure.\n"
       "Press RET at the end (ALT+RET to store into RAM only) : "
@@ -412,7 +417,7 @@ wchar_t ocr_db(struct box *box1) {
     fgets((char *)buf,200,stdin); /* including \n=0x0a */
     dd=strlen((char *)buf);
     /* output hexcode if verbose set */
-    if (JOB->cfg.verbose) {
+    if (job->cfg.verbose) {
       fprintf(stderr, "\n# fgets [%d]:", dd);
       for(i=0; i<dd; i++)
         fprintf(stderr, " %02x", (unsigned)((unsigned char)buf[i]));
@@ -421,7 +426,7 @@ wchar_t ocr_db(struct box *box1) {
     /* we dont accept chars which could destroy database file */
     for (i=0; i<dd; i++) if (buf[i]<32) break; /* need unsigned char here */
     endchar=buf[i]; /* last char is 0x0a (ret) 0x00 (EOF) or 0x1b (alt+ret) */
-    if (endchar==0x01) { i=0;JOB->cfg.mode&=~128; } /* skip all */
+    if (endchar==0x01) { i=0;job->cfg.mode&=~128; } /* skip all */
     buf[dd=i]=0; /* replace final 0x0a or other special codes */
     if (dd==1 && !(buf[0]&128)) { c=buf[0]; utf8_ok=1; } /* single char */
     if (dd>1 && dd<7) {        /* try to decode single wide char (utf8) */ 
@@ -448,12 +453,12 @@ wchar_t ocr_db(struct box *box1) {
        */
       if (endchar!=0x01){ /* ^a before hit return */
       /* is there a reason to dont store to memory? */
-        list_app(&JOB->tmp.dblist, box1); /* append to list for 1+2 */
+        list_app(&job->tmp.dblist, box1); /* append to list for 1+2 */
       }
       if (endchar!=0x01 && endchar!=0x1b){
-        store_db(box1);  /* store to disk for 2 */
+        store_db(box1, job);  /* store to disk for 2 */
       }
-      if (JOB->cfg.verbose)
+      if (job->cfg.verbose)
         fprintf(stderr, " got  char= %c  16bit= 0x%04x  string= \"%s\"\n",
            ((c>32 && c<127)?(char)c:'.'), (int)c, buf);
     }
