@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000-2010 Joerg Schulenburg
+Copyright (C) 2000-2018 Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -67,9 +67,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
 {
   job_t *job=OCR_JOB; /* fixme global var */
-  int i, jj, j2, y, yy, my, mi, mc, i1, i2, i3, i4,
-      m1, m2, m3, m4, ma1, ma2, ma3, ma4, m3pre, m4pre;
-  struct box *box2, *box3;  /* box3 is for verbose / debugging */
+  int i, jj, /*j2,*/ y, yy, my, mi, /* mc,*/ i1, i2, i3, i4,
+      m1, m2, m3, m4, ma1, ma2, ma3, ma4, m3pre, m4pre,
+      mt1, mt2, mt4, mtc; /* 2017 from global triple-pxH */
+  struct box *box2, *box3, *box4;  /* box3 is for verbose / debugging */
   struct tlines *lines = &job->res.lines;
 
   /* ToDo: optional read line-data from external source??? */
@@ -128,24 +129,77 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
    */
   my /= jj;          /* we only care about chars with high arround my */
   if (job->cfg.verbose & 16)
-    fprintf(stderr,"\n# detect_lines1(%d %d %d %d) vvv&16 chars=%d my=%d\n#  ",
+    fprintf(stderr,"\n# detect_lines1(%d %d %d %d) vvv&16 chars=%d my=%d\n#",
       x0, y0, dx, dy, jj, my);
   // "my" is the average over the whole image (bad, if different fontsizes)
 
   if (my < 4)
     return 0;       /* mean high is to small => error */
 
+  //  ToDo: find 2 neighbouring chars per line pH Hp + third x or p or H to verify
+  //     to set default heights
+#if 1
+  /* this is only for test runs */
+  if (job->cfg.verbose & 16)
+    fprintf(stderr," search pxH-like-triple using   my=%4d\n#",my);
+#endif
+  mt1= mt2= mt4= mtc= 0; /* averages m1..m4 to base-line m2 */
+  /* O(n^3)-loop = slow = bad */
+  for_each_data(&(job->res.boxlist)) { /* 2017-07 added , ToDo: use ma1-ma4 */
+    box2 = (struct box *)list_get_current(&(job->res.boxlist));
+    if (box2->c == PICTURE) continue;
+    for_each_data(&(job->res.boxlist)) {
+      box3 = (struct box *)list_get_current(&(job->res.boxlist));
+      if (box3->c == PICTURE) continue;
+      if (box2==box3) continue;
+      if (box3->y1 < box2->y0) continue; /* other above line */
+      if (box3->y0 > box2->y1 + 2*my) break;    /* other below line */
+      if (box3->x1 < box2->x0 - 8*my) continue; /* far away left char */
+      if (box3->x0 > box2->x1 + 8*my) continue; /* far away right char */
+      if (box2->y0 != box3->y0
+       || box2->y1 <= box3->y1) continue; /* px-like */
+      for_each_data(&(job->res.boxlist)) {
+        box4 = (struct box *)list_get_current(&(job->res.boxlist));
+        if (box4->c == PICTURE) continue;
+        if (box2==box4 || box3==box4) continue;
+        if (box4->y1 < box2->y0) continue; /* other above line */
+        if (box4->y0 > box2->y1 + 2*my) break;    /* other below line */
+        if (box4->x1 < box2->x0 - 8*my) continue; /* far away left char */
+        if (box4->x0 > box2->x1 + 8*my) continue; /* far away right char */
+        if (box3->y1 != box4->y1
+         || box3->y0 <= box4->y0) continue; /* pxH-like */
+//        if (job->cfg.verbose & 16)
+//          fprintf(stderr,"  found pH-like-pair xy %3d %3d  %3d %3d  %3d %3d\n#",
+//                  box2->x0,box2->y0,box3->x0,box3->y0,box4->x0,box4->y0);
+        mtc++;
+        mt1+= box4->y0 - box4->y1; /* neg. H-line */
+        mt2+= box3->y0 - box3->y1; /* neg. x-line */
+        mt4+= box2->y1 - box3->y1; /* pos. p-line */
+      } end_for_each(&(job->res.boxlist));
+    } end_for_each(&(job->res.boxlist));
+  } end_for_each(&(job->res.boxlist));
+  if (mtc==0) {mt1=mt2=mt4=0;mtc=1;}
+  if (job->cfg.verbose & 16)
+    fprintf(stderr,"  found %4d pxH-like-triples m124=%4.1f %4.1f %4.1f\n#",
+            mtc, (mt1*1.)/mtc, (mt2*1.)/mtc, (mt4*1.)/mtc);
+  if (job->cfg.verbose & 16)
+    fprintf(stderr,"  found %4d pxH-like-triples m124=%4d %4d %4d\n#",
+            mtc, (mt1-mtc+1)/mtc, (mt2-mtc+1)/mtc, (mt4+mtc-1)/mtc);
+  mt1=(mt1-mtc+1)/mtc; /* round up! */
+  mt2=(mt2-mtc+1)/mtc;
+  mt4=(mt4+mtc-1)/mtc; /* added 2017-07 ToDo: use for short lines later */
+
   m4pre=m3pre=y0;   /* lower bond of upper line */
-  // better function for scanning line around a letter ???
-  // or define lines around known chars "eaTmM"
-  for (j2 = y = y0; y < y0 + dy; y++) {
+  // ToDo: better function for scanning line around a letter ???
+  //       or define lines around known chars "eaTmM"
+  for (/*j2 =*/ y = y0; y < y0 + dy; y++) { /* line by line loop */
     // look for max. of upper and lower bound of next line
     m1 = y0 + dy;
     jj = 0;
 #if 1
     /* this is only for test runs */
     if (job->cfg.verbose & 16)
-      fprintf(stderr,"searching new line %d\n#  ",i /* lines->num */);
+      fprintf(stderr," searching new line %d\n#",i /* lines->num */);
 #endif
 
     box3 = NULL; /* mark the most upper box starting next line */
@@ -169,22 +223,27 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
        {
 	  if (box2->y0 < m1 + yy) {
 	    m1 = box2->y0 - yy;  /* highest upper boundary */
-	    box3 = box2;
-	  }
+	    box3 = box2; }
 	  // fprintf(stderr,"\n %3d %3d %+3d %d m1= %3d",
           //   box2->x0, box2->y0, box2->y1 - box2->y0 + 1, box2->num_boxes, m1);
        }
     } end_for_each(&(job->res.boxlist));
     if (!box3 || m1 >= y0+dy) break; /* no further line found */
     if (job->cfg.verbose & 16)
-      fprintf(stderr," most upper box at new line xy= %4d %4d %+4d %+4d\n#  ",
+      fprintf(stderr," most upper box at new line xy= %4d %4d %+4d %+4d\n#",
        box3->x0, box3->y0, box3->x1-box3->x0, box3->y1-box3->y0);
 
     // at the moment values depend from single chars, which can 
     //    result in bad values (ex: 4x6 /\=)
+    //    ToDo: better use triples with 2 chars same line, one above or below
     // ToDo: 2) mean size of next line (store list of y0,y1)
     // ToDo: 3) count num0[(y0-m1)*16/my], num1[(y1-m1)*16/my]
     // ToDo: or down-top search horizontal nerarest neighbours
+    // ToDo: histogramm of highs + shrink to 3-4 main-peaks
+    //       start histogramm with tripples xHx or xpx and get p-length
+    //       assuming constant font + certainty?
+    //       p_len[4]={} p_line-base_line + certainty[4]={0,0,0,0}
+    //     what is the simplest good way?
     lines->x0[i] = x0 + dx - 1; /* expand during operation to left end */
     lines->x1[i] = x0;          /* expand to the right end of line */
     m4=m2=m1; mi=m1+my; m3=m1+2*my; jj=0;
@@ -234,13 +293,13 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
         if (box2->y0 > m2 + yy) {
           m2 = box2->y0 - yy;  /* highest upper boundary */
           if (job->cfg.verbose & 16)
-            fprintf(stderr," set m2= %3d %+3d yy= %d  reason1 xy= %4d %4d\n#  ",
+            fprintf(stderr,"  set m2= %3d %+3d yy= %2d  reason1 xy= %4d %4d\n#",
                     m2, m2-m1, yy, box2->x0, box2->y0);
         }
         if (box2->y1 > m4 + yy && (my>6 || box2->y1 < m3+my)) {
           m4 = box2->y1 - yy;  /* lowest lower boundary, small font lines can touch */
           if (job->cfg.verbose & 16)
-            fprintf(stderr," set m4= %3d %+3d yy= %d  reason1 xy= %4d %4d\n#  ",
+            fprintf(stderr,"  set m4= %3d %+3d yy= %2d  reason1 xy= %4d %4d\n#",
                     m4, m4-m1, yy, box2->x0, box2->y0);
         }
         if (   box2->y1 < m3 + yy
@@ -254,7 +313,7 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
         {
           m3 = box2->y1 - yy;  /* highest lower boundary */
           if (job->cfg.verbose & 16)
-            fprintf(stderr," set m3= %3d %+3d yy= %d  reason1 xy= %4d %4d\n#  ",
+            fprintf(stderr,"  set m3= %3d %+3d yy= %2d  reason1 xy= %4d %4d\n#",
                     m3, m3-m1, yy, box2->x0, box2->y0);
           // printf("\n# set1 m3 m=%3d %+2d %+2d %+2d",m1,m2-m1,m3-m1,m4-m1);
           // out_x(box2);
@@ -266,7 +325,7 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
         {
           m3 = box2->y1 - yy;  /* highest lower boundary */
           if (job->cfg.verbose & 16)
-            fprintf(stderr," set m3= %3d %+3d yy= %d  reason2 xy= %4d %4d\n#  ",
+            fprintf(stderr,"  set m3= %3d %+3d yy= %d  reason2 xy= %4d %4d\n#",
                     m3, m3-m1, yy, box2->x0, box2->y0);
           // printf("\n# set2 m3 m=%3d %+2d %+2d %+2d",m1,m2-m1,m3-m1,m4-m1);
           // out_x(box2);
@@ -274,7 +333,8 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
       } // good middle size char
     } end_for_each(&(job->res.boxlist));
 
-    // ToDo: use holes for m1==m2 and m3==m4
+    // ToDo: use holes for m1==m2 and m3==m4 pqbdoaegABDOPQR0689%&#
+    // or ij-dots + umlauts + .,;?! ??
     if (m3==m4 && i>1) { // use predecessor 2010-10-01
       if (m3-m1==lines->m3[i-1]-lines->m1[i-1])
           m4= m1+lines->m4[i-1]-lines->m1[i-1];
@@ -288,17 +348,32 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
 #if 1
     /* this is only for test runs */
     if (job->cfg.verbose & 16)
-      fprintf(stderr," L%02d step 1 y=%4d m= %4d %+3d %+3d %+3d x= %3d %+3d"
-                     " my=%2d chars=%3d\n#  ",
+      fprintf(stderr,"  L%02d step 1 y=%4d m= %4d %+3d %+3d %+3d x= %3d %+3d"
+                     " my=%2d chars=%3d\n#",
                      i, y, m1, m2-m1, m3-m1, m4-m1, lines->x0[i], 
                      lines->x1[i] - lines->x0[i], my, jj);
 #endif
 
-    if (m3 == m1)
-      break;
+    if (m3 == m1) break;
+    
+    
+    
+    /* ToDo17 correct m3==m4 or m1==m2 using mt1..mt4 */
+
+    
+#if 1
+    /* this is only for test runs */
+    if (job->cfg.verbose & 16)
+      fprintf(stderr,"  L%02d step 1 y=%4d m= %4d %+3d %+3d %+3d x= %3d %+3d"
+                     " my=%2d chars=%3d\n#",
+                     i, y, m1, m2-m1, m3-m1, m4-m1, lines->x0[i], 
+                     lines->x1[i] - lines->x0[i], my, jj);
+#endif
+    
+    
 #if 1         /* make averages about the line  */
     // same again better estimation
-    mc = (3 * m3 + m1) / 4;	/* lower center ? */
+    // mc = (3 * m3 + m1) / 4;	/* lower center ? */
     ma1 = ma2 = ma3 = ma4 = i1 = i2 = i3 = i4 = jj = 0;
     for_each_data(&(job->res.boxlist)) {
       box2 = (struct box *)list_get_current(&(job->res.boxlist));
@@ -326,10 +401,11 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
       }
     } end_for_each(&(job->res.boxlist));
 
-    if (i1)  m1 = (ma1+i1/2) / i1; /* best rounded */
-    if (i2)  m2 = (ma2+i2/2) / i2;
-    if (i3)  m3 = (ma3+i3-1) / i3; /* round up */
-    if (i4)  m4 = (ma4+i4-1) / i4;
+    // tmp17/0506_clean7x9c.png dbtlhk lower than ABCD
+    if (i1)  m1 = (ma1+i1/4) / i1; /* best rounded, H-line */
+    if (i2)  m2 = (ma2+i2/2) / i2; /* x-line */
+    if (i3)  m3 = (ma3+i3-1) / i3; /* round up, base-line */
+    if (i4)  m4 = (ma4+i4-1) / i4; /* p-line */
     // printf("\n# .. set3 m3 m=%3d %+2d %+2d %+2d",m1,m2-m1,m3-m1,m4-m1);
 
 #endif
@@ -357,9 +433,9 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
     } end_for_each(&(job->res.boxlist));
 
 #if 1
-    /* this is only for test runs */
+    /* this is only for test runs, m1=H-line m2=x-line m3=base m4=p-line */
     if (job->cfg.verbose & 16)
-      fprintf(stderr," L%02d step 2 y=%4d m= %4d %+3d %+3d %+3d x= %3d %+3d\n#  ",
+      fprintf(stderr,"  L%02d step 2 y=%4d m= %4d %+3d %+3d %+3d x= %3d %+3d\n#",
         i, y, m1, m2-m1, m3-m1, m4-m1,
         lines->x0[i], lines->x1[i]-lines->x0[i]);
 #endif
@@ -375,18 +451,34 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
     if (jj&1) lines->wt[i] = 75*lines->wt[i]/100;
     if (jj&2) lines->wt[i] = 75*lines->wt[i]/100;
     if (jj>0 && job->cfg.verbose) {
-      fprintf(stderr," L%02d w= %3d y= %3d m= %4d %+3d %+3d %+3d\n#  ",i,lines->wt[i],y,m1,m2-m1,m3-m1,m4-m1);
-      fprintf(stderr," L%02d        y= %3d i=  %3d %3d %3d %3d (counts)\n#  ",i,y,i1,i2,i3,i4);
-      if (jj==3) fprintf(stderr," all boxes of same high! mi=%d\n#  ",mi-m1);
-      if (jj==1) fprintf(stderr," all boxes of same upper bound!\n#  ");
-      if (jj==2) fprintf(stderr," all boxes of same lower bound!\n#  ");
+      /* m1=cap_high=H-line m2=median=x-line m3=base m4=descent=p-line */
+      /* see https://en.wikipedia.org/wiki/Baseline_%28typography%29 */
+      fprintf(stderr,"  L%02d w= %3d y= %3d m= %4d %+3d %+3d %+3d (H,x,base,p)\n#",i,lines->wt[i],y,m1,m2-m1,m3-m1,m4-m1);
+      fprintf(stderr,"  L%02d        y= %3d i=  %3d %3d %3d %3d (counts)\n#",i,y,i1,i2,i3,i4);
+      if (jj==3) fprintf(stderr,"  all boxes of same high! mi=%d\n#",mi-m1);
+      if (jj==1) fprintf(stderr,"  all boxes of same upper bound!\n#");
+      if (jj==2) fprintf(stderr,"  all boxes of same lower bound!\n#");
      }
     /* ToDo: check for dots ij,. to get the missing information */
 #if 1
     // ToDo: do this after all lines are detected using info of other lines?
+    //  ToDo17 use Hxp-triple values mt1,mt2,mt4 to fix jj!=0 lines
+    //         sample: tmp17/0506_clean7x9a.png (short lines)
     //  or use all chars to solve problem cases
     if (mi<m1-(m4-m1)) mi=0; // mi failed
     /* jj=3: ABCDEF123456 or mnmno or gqpy or lkhfdtb => we are in trouble */
+    if (mt1<mt2 && mt2<0 && mt4>0 && jj==3) { /* valid Hxp-tripples found, use here */
+      fprintf(stderr,"  trying pxH-fix by Hxp %3d %3d   0 %3d\n#",
+            mt1, mt2, mt4);
+      if (m4-m1+mt2<=my/8) /* acemnorsuvwxz */
+        { jj=0; m1=m3+mt1; m4=m3+mt4; } /* 2017-08 */
+      if (m4-m1+mt1<=my/8) /* ABCDEF */
+        { jj=0; m2=m3+mt2; m4=m3+mt4; } /* 2017-08 but mt1 == mt4-mt2? */
+      if (m4-m1+mt2-mt4<=my/8) /* pqy */
+        { jj=0; m1=m2-mt2+mt1; m3=m2-mt2; } /* 2017-08 */
+      if (jj==0) fprintf(stderr,"       fixed by Hxp to %4d %3d %3d %3d\n#",
+            m1, m2-m1, m3-m1, m4-m1);
+    }
     /* 2010-10-06 my/8 to my/5, see examples/ocr-b */
     if (jj==3 && (m4-m1)>=my) { jj=0; m2=m1+my/5+1; m4=m3+my/5+1; } /* ABC123 */
     /* using idots, may fail on "ABCDEFG&Auml;&Uuml;&Ouml;" */
@@ -404,21 +496,21 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
     if (jj&2)
       m4 = m3 + (m4 - m1) / 4 + 1;	/* chars have same lower base */
     if (jj>0 && job->cfg.verbose & 16) {
-      fprintf(stderr," L%02d jj %3d y= %3d m= %4d %+3d %+3d %+3d  my= %4d\n#  ",
+      fprintf(stderr,"  L%02d jj %3d y= %3d m= %4d %+3d %+3d %+3d  my= %4d\n#",
               i, jj, y, m1, m2-m1, m3-m1, m4-m1, my);
      }
 #endif
 
     
     {				// empty space between lines
-      lines->m4[i] = m4;
-      lines->m3[i] = m3;
-      lines->m2[i] = m2;
-      lines->m1[i] = m1;
+      lines->m4[i] = m4; // p-line (descent)
+      lines->m3[i] = m3; // base-line
+      lines->m2[i] = m2; // x-line (median)
+      lines->m1[i] = m1; // H-line
       lines->pitch[i] = job->cfg.spc; /* default word pitch */
       lines->mono[i] = 0;             /* default spacing, 0=prop, 1=mono */
       if (job->cfg.verbose & 16)
-	fprintf(stderr, " L%02d final  y= %3d m= %4d %+3d %+3d %+3d x= %3d %+3d w= %d\n#  ",
+	fprintf(stderr, "  L%02d final  y= %3d m= %4d %+3d %+3d %+3d x= %3d %+3d w= %d\n#",
 		i, y, m1, m2 - m1, m3 - m1, m4 - m1, lines->x0[i],
 		lines->x1[i] - lines->x0[i], lines->wt[i]);
       if (i < MAXlines && m4 - m1 > 4)
@@ -434,7 +526,7 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
   }
   lines->num = i;
   if (job->cfg.verbose)
-    fprintf(stderr, " num_lines= %d", lines->num-1);
+    fprintf(stderr, "# detect_lines1.num_lines= %d", lines->num-1);
   return 0;
 }
 
@@ -755,10 +847,9 @@ int detect_text_lines(pix * pp, int mo) {
   if (mo & 4){
     if (vvv) fprintf(stderr, "# zoning\n# ... ");
     detect_lines2(pp, 0, 0, pp->x, pp->y, 0);	// later replaced by better algo
+    if (vvv) fprintf(stderr,"\n");
   } else 
     detect_lines1(pp, 0, 0, pp->x, pp->y);	// old algo
-
-  if (vvv) fprintf(stderr,"\n");
   return 0;
 }
 
