@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) GPLv2 2000-2013  Joerg Schulenburg
+Copyright (C) GPLv2 2000-2018  Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -41,12 +41,13 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "pcx.h"
 #include "ocr0.h"		/* only_numbers */
 #include "progress.h"
-#include "version.h"
+#include "version.h" /* include/version.h version_string + release_string */
 
+/* v+=2 exit, v+=1 add_how_to_get_help */
 static void out_version(int v) {
   fprintf(stderr, " Optical Character Recognition --- gocr "
           version_string " " release_string "\n"
-          " Copyright (C) 2001-2010 Joerg Schulenburg  GPG=1024D/53BDFBE3\n"
+          " Copyright (C) 2001-2018 Joerg Schulenburg  GPG=1024D/53BDFBE3\n"
           " released under the GNU General Public License\n");
   /* as recommended, (c) and license should be part of the binary */
   /* no email because of SPAM, see README for contacting the author */
@@ -63,7 +64,7 @@ static void help(void) {
   fprintf(stderr,
 	  " using: gocr [options] pnm_file_name  # use - for stdin\n"
 	  " options (see gocr manual pages for more details):\n"
-	  " -h, --help\n"
+	  " -h, --help, -V --version\n"
 	  " -i name   - input image file (pnm,pgm,pbm,ppm,pcx,...)\n"
 	  " -o name   - output file  (redirection of stdout)\n"
 	  " -e name   - logging file (redirection of stderr)\n"
@@ -87,8 +88,8 @@ static void help(void) {
 	  "\tgocr -m 130 -p ./database/ text1.pbm  # extend database\n"
 	  "\tdjpeg -pnm -gray text.jpg | gocr -    # use jpeg-file via pipe\n"
 	  "\n");
-  fprintf(stderr, " webpage: http://jocr.sourceforge.net/ (may out of date)\n");
-  fprintf(stderr, " mirror:  http://www-e.uni-magdeburg.de/jschulen/ocr/\n");
+  fprintf(stderr, " website: http://www-e.uni-magdeburg.de/jschulen/ocr/\n");
+/* fprintf(stderr, " website: http://jocr.sourceforge.net/ (2000-2013)\n"); */
   exit(0);
 }
 
@@ -136,20 +137,22 @@ static void process_arguments(job_t *job, int argn, char *argv[])
   
   /* process arguments */
   for (i = 1; i < argn; i++) {
-    if (strcmp(argv[i], "--help") == 0)
-      help(); /* and quits */
+    int iskip= 0; /* 2017-03-26 allow -v33 (iskip=0) or -v 33 (iskip=1) */
+    if (strcmp(argv[i], "--help") == 0
+     || strcmp(argv[i], "-h")     == 0) help(); /* and quits  2018-09 -h */
+    if (strcmp(argv[i], "--version") == 0
+     || strcmp(argv[i], "-V")        == 0) { /* and quits  2018-09 */
+       printf(version_string "-" release_string "\n"); exit(0);}
     if (argv[i][0] == '-' && argv[i][1] != 0) {
-      s1 = "";
-      if (i + 1 < argn)
-	s1 = argv[i + 1];
+      if (i + 1 < argn && argv[i][2]=='\0') { s1 = argv[i + 1]; iskip=1; }
+      else if (argv[i][2]!='\0')              s1 = argv[i]+2;
+           else {
+             fprintf(stderr,"ERROR: missing argument, try --help\n");
+             exit(1);
+           }
       switch (argv[i][1]) {
-      case 'h': /* help */
-	help();
-	break;
       case 'i': /* input image file */
-	job->src.fname = s1;
-	i++;
-	break;
+	job->src.fname = s1; break;
       case 'e': /* logging file */
 	if (s1[0] == '-' && s1[1] == '\0') {
 #ifdef HAVE_UNISTD_H
@@ -161,19 +164,14 @@ static void process_arguments(job_t *job, int argn, char *argv[])
 	else if (!freopen(s1, "w", stderr)) {
 	  fprintf(stderr, "stderr redirection to %s failed\n", s1);
 	}
-	i++;
 	break;
       case 'p': /* database path */
-	job->cfg.db_path=s1;
-	i++;
-	break;
+	job->cfg.db_path=s1; break;
       case 'o': /* output file */
-	if (s1[0] == '-' && s1[1] == '\0') {	/* default */
-	}
+	if (s1[0] == '-' && s1[1] == '\0') { } /* default = stdout */
 	else if (!freopen(s1, "w", stdout)) {
 	  fprintf(stderr, "stdout redirection to %s failed\n", s1);
-	};
-	i++;
+	}
 	break;
       case 'f': /* output format */
         if (strcmp(s1, "ISO8859_1") == 0) job->cfg.out_format=ISO8859_1; else
@@ -187,53 +185,34 @@ static void process_arguments(job_t *job, int argn, char *argv[])
         i++;
         break;
       case 'c': /* list of chars (_ = not recognized chars) */
-	job->cfg.lc = s1;
-	i++;
-	break;
+	job->cfg.lc = s1; break;
       case 'C': /* char filter, default: NULL (all chars) */
         /* ToDo: UTF8 input, wchar */
-	job->cfg.cfilter = s1;
-	i++;
-	break;
+	job->cfg.cfilter = s1; break;
       case 'd': /* dust size */
-	job->cfg.dust_size = atoi(s1);
-	i++;
-	break;
+	job->cfg.dust_size = atoi(s1); break;
       case 'l': /* grey level 0<160<=255, 0 for autodetect */
-	job->cfg.cs = atoi(s1);
-	i++;
-	break;
+	job->cfg.cs = atoi(s1); break;
       case 's': /* spacewidth/dots (0 = autodetect) */
-	job->cfg.spc = atoi(s1);
-	i++;
-	break;
+	job->cfg.spc = atoi(s1); break;
       case 'v': /* verbose mode */
-	job->cfg.verbose |= atoi(s1);
-	i++;
-	break;
+	job->cfg.verbose |= atoi(s1); break;
       case 'm': /* operation modes */
-	job->cfg.mode |= atoi(s1);
-	i++;
-	break;
+	job->cfg.mode |= atoi(s1); break;
       case 'n': /* numbers only */
-	job->cfg.only_numbers = atoi(s1);
-	i++;
-	break;
+	job->cfg.only_numbers = atoi(s1); break;
       case 'x': /* initialize progress output s1=fname */
-	ini_progress(s1);
-	i++;
-	break;
+	ini_progress(s1); break;
       case 'a': /* set certainty */
-	job->cfg.certainty = atoi(s1);;
-	i++;
-	break;
+	job->cfg.certainty = atoi(s1); break;
       case 'u': /* output marker for unrecognized chars */
-        job->cfg.unrec_marker = s1;
-        i++;
-        break;
+        job->cfg.unrec_marker = s1; break;
       default:
-	fprintf(stderr, "# unknown option use -h for help\n");
-      }
+	fprintf(stderr, "ERROR: unknown option %s, use -h for help\n",
+	        argv[i]);
+	exit(1); /* 2018-09 add exit on error */
+      } /* switch(argv[i][1]) */
+      i += iskip; /* 2017-03 add optional -v33 for -v 33 */
       continue;
     }
     else /* argument can be filename v0.2.5 */ if (argv[i][0] != '-'
