@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000-2010 Joerg Schulenburg
+Copyright (C) 2000-2018 Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -296,7 +296,7 @@ void store_boxtree_lines(job_t *job, int mo) {
         if (box2->c >  ' ' &&
             box2->c <= 'z') i2++; /* count non-space chars */
       } else { /* c == UNKNOWN or 0 */
-        wchar_t cc; cc=box2->c;
+        // wchar_t cc; cc=box2->c;   2018-09 unused
         if (box2->num_ac>0 && box2->tas[0]
           && (/* job->cfg.out_format!=XML || 2010-10 */ box2->tas[0][0]!='<')
           ) {  // 2010-10 output XML code after XML frame, see below!
@@ -322,9 +322,10 @@ void store_boxtree_lines(job_t *job, int mo) {
             buffer=append_to_line(buffer,s1,&len);
             if (i1+1<box2->num_ac) buffer=append_to_line(buffer,",",&len);
           }
-          if (box2->num_ac>1)
+          if (box2->num_ac>0) // 2018-09 achars also for 1++ chars, old 2++
             buffer=append_to_line(buffer,"\" achars=\"",&len);
-          for (i1=1;i1<box2->num_ac;i1++) {
+          // 2018-09 add char 0 for simpler parsing, old: i1=1,...
+          for (i1=0;i1<box2->num_ac;i1++) {
             if (box2->tas[i1] && box2->tas[i1][0]!='<')
               buffer=append_to_line(buffer,box2->tas[i1],&len);
             else
