@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000-2006  Joerg Schulenburg
+Copyright (C) 2000-2010  Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -211,7 +211,7 @@ int list_higher_level( List *l ) {
     l->level++;
     l->current[l->level] = l->start.next;
   }
-  g_debug(fprintf(stderr, " level++=%d current[]=%p\n",
+  g_debug(fprintf(stderr, "LEV5: level++=%d current[]=%p\n",
      l->level, l->current);)
   if ( !newcur  ) {
     fprintf(stderr, " realloc failed! abort\n"); return(1);
@@ -230,7 +230,7 @@ void list_lower_level( List *l ) {
     l->current = (Element **)realloc(l->current, l->level*sizeof(Element *));
   }
   l->level--;
-  g_debug(fprintf(stderr, " level--=%d current[]=%p\n", l->level, 
+  g_debug(fprintf(stderr, "LEV5: level--=%d current[]=%p\n", l->level, 
       l->current);)
 }
 
@@ -309,7 +309,7 @@ void list_sort( List *l, int (*compare)(const void *, const void *) ) {
   }
 
   close_progress(pc);
-  g_debug(fprintf(stderr, "list_sort()\n");)
+  g_debug(fprintf(stderr, "LEV3: list_sort()\n");)
 }
 
 /* calls free_data() for each data in list l, 
@@ -327,7 +327,7 @@ int list_and_data_free( List *l, void (*free_data)(void *data)) {
   
   list_free(l);
 
-  g_debug(fprintf(stderr, "list_and_data_free()\n");)
+  g_debug(fprintf(stderr, "LEV3: list_and_data_free()\n");)
 
   return 0;
 }
