@@ -1,7 +1,7 @@
 /* ocr-engine numbers only */
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000-2010 Joerg Schulenburg
+Copyright (C) 2000-2018 Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -34,36 +34,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "gocr.h"
 #include "unicode_defs.h"
 
-/* only for debugging and development */
-#define IFV if(OCR_JOB->cfg.verbose&4)
-#define MM {IFV fprintf(stderr,"\nDBG %c L%04d (%d,%d): ",(char)c_ask,__LINE__,box1->x0,box1->y0);}
-
-/* the old debug mode (0.40) was only for a special char, for another char
- *   code must be recompiled with C_ASK='char'
- * new debug mode (0.41) explains why char is declined or accepted as ABC...
- *   the output can be filtered by external scripts
- *   ToDo: we could reduce output to filter string
- */
-#ifndef DO_DEBUG       /* can be defined outside */
-#define DO_DEBUG 0     /* 0 is the default */
-#endif
-
-/* this macro is for debugging output: "if char is declined, why?" */
-#if DO_DEBUG   /* 0=Work mode, 1=debugging mode */
-// Setac: output, that char is choosen with a probability
-// Break: output, why the char is not choosen
-// MSG: debugging functions for char C_ASK, mostly messages
-// DBG: definitions usefull only for debugging
-#define Setac(box1,ac,ad)  { MM;IFV fprintf(stderr,"setac %d",ad);setac(box1,ac,ad); }
-#define Break              { MM;IFV fprintf(stderr,"break"); break; }
-#define MSG(x)             { MM;IFV x }
-#define DBG(x) x
-#else
-#define Setac(box1,ac,ad)  setac(box1,ac,ad)
-#define Break break
-#define MSG(x)
-#define DBG(x)
-#endif
+#include "ocr0_dbg.h" /* define DO_DEBUG IFV MM Setac Break MSG DBG */
 
 /* extern "C"{ */
 
@@ -76,7 +47,7 @@ wchar_t ocr0n(ocr0_shared_t *sdata){
    int  xa,xb,ya,yb, /* tmp-vars */
         i1,i2,i3,i4,i,j;
    int (*aa)[4]=sdata->aa;    /* corner-points, (x,y,dist^2,vector_idx) */
-   wchar_t bc=UNKNOWN;				// best char
+   //wchar_t bc=UNKNOWN;				// best char (obsolete)
    int  ad=0;		// propability 0..100
    int hchar=sdata->hchar;	// char is higher than 'e'
    int gchar=sdata->gchar;	// char has ink lower than m3
@@ -104,7 +75,8 @@ wchar_t ocr0n(ocr0_shared_t *sdata){
       // get the upper and lower hole koords, y around dy/4 ???
       x=5*dx/8;
       y  =loop(bp,x,0,dy,cs,0,DO); if(y>dy/8) Break;
-      y +=loop(bp,x,y,dy,cs,1,DO); if(y>(dy+2)/4) Break; // 6x7 2010-09-25
+      y +=loop(bp,x,y,dy,cs,1,DO); if(y>(dy+2)/4) ad=98*ad/100; // 2018-09 815
+                                   if(y>(dy+2)/3) Break; // 6x7 2010-09-25
       i1 =loop(bp,x,y,dy,cs,0,DO)+y; if(i1>5*dy/8) Break;
       i3=y=(y+i1)/2; // upper end can be shifted to the right for italic
       x  =loop(bp,0,y,dx,cs,0,RI); if(x>4*dx/8) Break;
@@ -271,7 +243,7 @@ wchar_t ocr0n(ocr0_shared_t *sdata){
       x =loop(bp,(i1+i2)/2,dy/2,dx,cs,1,RI); i=x; i3=0;
       for (y=0;y<7*dy/8;y++)
         if ( loop(bp,i1+y*(i2-i1)/dy, y,dx,cs,1,RI)-i > 1+dx/8 ) break;
-      if (y<7*dy/8) { ad=98*ad/100; // serif or ocr-a-1 ?
+      if (y<7*dy/8) { ad=99*ad/100; // serif or ocr-a-1 ?
         MSG(fprintf(stderr,"i12 %d %d ad= %d", i1,i2,ad);) }
       if (y<6*dy/8) { ad=99*ad/100; /*  MICR E-13B font Jan07 */
         MSG(fprintf(stderr,"i12 %d %d ad= %d", i1,i2,ad);) }
@@ -286,6 +258,18 @@ wchar_t ocr0n(ocr0_shared_t *sdata){
       if(i3) Break;        
 //       out_x(box1);printf(" 11 i=%d j=%d i2=%d dx=%d\n",i,j,i1,dx);
       MSG(fprintf(stderr,"ad= %d", ad);)
+      // check against v-symmetric I of examples/5x7.png 2018-09
+      for(y=0;y<dy;y++){
+        x =loop(bp,0   ,y,dx,cs,0,RI);
+        j =loop(bp,dx-1,y,dx,cs,0,LE); if(x!=j) break;
+      } if (y==dy) { ad=96*ad/100; MSG({}) } // more an (v-symmetric) I
+      // check against h-symmetric () of examples/5x7.png 2018-09
+      for(x=0;x<dx;x++){
+        y =loop(bp,x,0   ,dy,cs,0,DO);
+        j =loop(bp,x,dy-1,dy,cs,0,UP); if(y!=j) break;
+        y+=loop(bp,x,y     ,dy,cs,1,DO);
+        j+=loop(bp,x,dy-1-j,dy,cs,1,UP); if(y!=j) break;
+      } if (x==dx) { ad=96*ad/100; MSG({}) }// more an (h-symmetric) ()
       // get most left upper point (i,j)
       for(i=dx,j=y=0;y<7*dy/16;y++){
         x =loop(bp,0,y,dx,cs,0,RI); if(x<i) { i=x;j=y; }
@@ -332,7 +316,7 @@ wchar_t ocr0n(ocr0_shared_t *sdata){
       for (y=dy,x=dx/2;x<3*dx/4;x++){ /* get upper end y */
         j=loop(bp,x,0,dy,cs,0,DO); if (j<y) { y=j; }
       }
-      if(y<dy/2 && y+dy/16>=i) ad=98*ad/100; // ~\tt l ??? ocr-a_1
+      if(y<dy/2 && y+dy/16>=i) ad=99*ad/100; // ~\tt l ??? ocr-a_1
       MSG(fprintf(stderr,"1l check i= %d y= %d ad= %d", i, y, ad);)
 
       if(   loop(bp,   0,  dy/8,dx,cs,0,RI)
@@ -726,7 +710,7 @@ wchar_t ocr0n(ocr0_shared_t *sdata){
    // --- test 4 --------------------------------------------------- 25Nov06
    for(ad=d=100;dy>3 && dx>2;){     // min 3x4 ~<gA',
      // rewritten for vectors 0.42
-      int ld, i1, i2, i3, i4, i5, i6, i7, i2up;  // line derivation + corners
+      int ld, i1, i2, i3, i4, i5, i6, i7, i2up, i2dwn;  // line derivation + corners
       DBG( wchar_t c_ask='4'; )
       if (sdata->holes.num > 1) Break; /* no or one hole */
       /* half distance to the center */
@@ -762,6 +746,8 @@ wchar_t ocr0n(ocr0_shared_t *sdata){
       i2=nearest_frame_vector(box1, 0, i, x0-2*dx, (y0+7*y1)/8);
       i2up=
          nearest_frame_vector(box1, 0, i, x0-2*dx, (5*y0+3*y1)/8);
+      i2dwn=
+         nearest_frame_vector(box1, 0, i, x0-4, y1+4);
       /* lowest vector */
       i4=nearest_frame_vector(box1, 0, i, (x0+2*x1)/3, y1+dy);
       /* right center crossing point */     
@@ -804,6 +790,19 @@ wchar_t ocr0n(ocr0_shared_t *sdata){
       MSG(fprintf(stderr,"i1-7 %d %d %d %d %d %d %d 2up %d",i1,i2,i3,i4,i5,i6,i7,i2up);)
       if (i5==i6) Break; // ~+
       
+      if (sdata->holes.num == 1    // handwritten 'a' ? 2018-09
+        && box1->frame_vector[i1][0] <= x0+2*dx/3 // left upper corner
+        && box1->frame_vector[i1][1] <= y0+  dy/8
+        && (!hchar)
+      ) { ad=96*ad/100; MSG(fprintf(stderr,"more an (a)? ad= %d",ad);)}
+      
+      MSG(fprintf(stderr,"i2dwn=%d",i2dwn);)
+      if (sdata->holes.num == 0    // handwritten 'u' ? 2018-09
+        && box1->frame_vector[i2dwn][0] <= x0+dx/2 // left upper corner
+        && box1->frame_vector[i2dwn][1] >= y1-dy/8
+        && (!hchar)
+      ) { ad=96*ad/100; MSG(fprintf(stderr,"more an (u)? ad= %d",ad);)}
+      
       if (box1->frame_vector[i1][1]>y0+dy/8) Break; // not to low
       if (box1->frame_vector[i2][1]
          -box1->frame_vector[i1][1]<dy/2) Break;
@@ -975,7 +974,7 @@ wchar_t ocr0n(ocr0_shared_t *sdata){
       if (         !hchar) ad=99*ad/100;
       if (gchar && !hchar) ad=98*ad/100; // ~q
       Setac(box1,(wchar_t)'4',ad);      
-      if (ad>99) bc='4';
+      // if (ad>99) bc='4';
       break;
    }
 #endif
@@ -1041,13 +1040,26 @@ wchar_t ocr0n(ocr0_shared_t *sdata){
       if (!hchar) ad=98*ad/100;
       if ( box1->dots ) ad=98*ad/100;
       Setac(box1,(wchar_t)'6',ad);
-      bc='6';
+      // bc='6';
       break;
    }
    // --- test 7 ---------------------------------------------------
    for(ad=d=100;dx>2 && dy>4;){     // dx>1 dy>2*dx
-      DBG( char c_ask='7'; )
-      if (sdata->holes.num > 1) Break; /* be tolerant */
+      DBG( wchar_t c_ask='7'; )
+      if (sdata->holes.num > 0) Break; /* no hole */
+      /* half distance to the center */
+      d=2*sq(128/4);
+      /* now we check for the lower ends, must be near to the corner */
+      if (aa[1][2]<d/4        
+       && aa[2][2]<d/4) Break;  /* [2] = distance, ~2... */
+      if (aa[0][2]>d/2) Break;  /* [2] = distance, ~d... */
+      if (aa[3][2]>d/2) Break;  /* [2] = distance, ~bhk... */
+      /* check for upper long line ~) 4x11 */
+      if (4*(aa[3][0]-aa[0][0])<3*dx) Break;
+      /* check for low end width ~2 2017-07 */
+      j=loop(bp,0,dy-1,dx,cs,0,RI);
+      j=loop(bp,j,dy-1,dx,cs,1,RI); if(3*j>dx) Break; // max thickness 1/3
+      /* searching for 4 notches between neighbouring ends */
       if( loop(bp,dx/2,0,dy,cs,0,DO)>dy/8 ) Break;
       if( num_cross(0,dx-1,3*dy/4,3*dy/4,bp,cs) != 1 ) Break; // preselect
       for( yb=xb=y=0;y<dy/2;y++){ // upper h-line and gap
@@ -1078,7 +1090,7 @@ wchar_t ocr0n(ocr0_shared_t *sdata){
       if( num_cross(0,dx-1,dy/4,dy/4,bp,cs) == 1
        && loop(bp,0,dy/4,dx,cs,0,RI) < dx/2 ) ad=ad*96/100; // J
       MSG( fprintf(stderr,"7: ad= %d",ad); )
-
+      /* check for upper long line ~) 4x11 */
       if (box1->m3 &&   dy<box1->m3-box1->m2) ad=99*ad/100; // too small
       if (box1->m3 && 2*dy<box1->m3-box1->m2) ad=96*ad/100; // too small
       if (dy>3*dx) ad=99*ad/100; // )
@@ -1163,8 +1175,8 @@ wchar_t ocr0n(ocr0_shared_t *sdata){
       if (num_cross(   0,dx-1,   1,   1,bp,cs) > 1) ad=98*ad/100;
       /* if m1..4 is unsure ignore hchar and gchar ~ga */
       if (!hchar) {
-        if ((box1->m2-box1->y0)*8>=dy) ad=98*ad/100;
-        else                           ad=99*ad/100;
+        if ((box1->m2-box1->y0)*8>=dy-dy/16) ad=98*ad/100;
+        else                                 ad=99*ad/100;
       }
       if ( gchar
          && (box1->y1-box1->m3)*8>=dy) ad=99*ad/100;
