@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000-2010  Joerg Schulenburg
+Copyright (C) 2000-2018  Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -209,8 +209,9 @@ int debug_img(char *fname, struct job_s *job, int opt) {
   struct box *box2;
   int x, y, ic, dx, i, j, col;
   unsigned char *np;
+  if (!job) { fprintf(stderr,"ERR job==NULL\n"); exit(1); }
   pix *pp = &job->tmp.ppo;
-
+  
   if ( opt & 8 ) {		/* clear debug bits in image */
     for(y=0;y<pp->y;y++) {
       np=&pp->p[(pp->x)*y];
@@ -222,7 +223,7 @@ int debug_img(char *fname, struct job_s *job, int opt) {
   }
 
   /* mark longest line which was used to estimate the rotation angle */
-  if ((job->cfg.verbose&32) && job->res.lines.dx)
+  if ((job->cfg.verbose&32) && job->res.lines.dx && (4&opt)/* 2018-10*/)
     for(i=0;i<pp->x;i++) {
       y=pp->y/2;
       if (job->res.lines.dx) y+=job->res.lines.dy*i/job->res.lines.dx;
@@ -232,10 +233,11 @@ int debug_img(char *fname, struct job_s *job, int opt) {
       if (*np<160) continue;
       if((x&7)<5 && !(x&1))  /* dotted line */
         put(pp,x,y,255,8);
-    }
+    } /* base line */
 
   ic = ((opt & 2) ? 1 : 2); /* obsolete */
-  for_each_data(&(job->res.boxlist)) {
+  if (3&opt) {
+  for_each_data(&(job->res.boxlist)) {  /* char boxes */
     box2 = (struct box *) list_get_current(&(job->res.boxlist));
     /* mark boxes in 32=0x40=blue */
     if (box2->c == ' ' || box2->c == '\n') continue;
@@ -255,37 +257,39 @@ int debug_img(char *fname, struct job_s *job, int opt) {
          pp->p[(box2->x0+x) + (box2->y0+y) * pp->x] |= 4;
          pp->p[(box2->x1-x) + (box2->y0+y) * pp->x] |= 4;
       }
-  } end_for_each(&(job->res.boxlist));
+  } end_for_each(&(job->res.boxlist)); /* char boxes */
+  } /* opt */
 
   if( opt & 4 )
   {
-      struct tlines *lines = &job->res.lines;
-      int yr;
-      if (job->cfg.verbose)                                          
-        fprintf(stderr, "# mark lines for %s.ppm\n", fname); 
-      /* or read from outside??? */
-      for (i = 0; i < lines->num; i++) { /* mark lines by 0x08 = blue */
-        dx = lines->x1[i] - lines->x0[i] + 1;
-	for (j = -1; j < dx+1; j++) {
-	  x  = lines->x0[i] + j;
-          if (x<0 || x>=pp->x) continue;
-	  for (y=lines->m1[i];y<=lines->m4[i];y++) {
-	    /* box arround m2-m3 */
-	    if (y>=lines->m2[i] && y<=lines->m3[i] && j>-1 && j<dx) continue;
-            yr  = y; /* y.rotated */
-            if (lines->dx) yr += lines->dy * x / (lines->dx);
-            if (yr<0 || yr>=pp->y) continue;
-            np = &(pp->p[x + (pp->x)*yr]);
-            if (*np<160) continue; /* do not touch dark pixels */
-            if ((*np&6)!=0) continue; /* only change white pixels */
-            put(pp, x, yr, 255, 6); /* UPN: 255 and 6 or */
-	  }
-	}
-      }
-  }
+    struct tlines *lines = &job->res.lines;
+    int yr;
+    if (job->cfg.verbose)                                          
+      fprintf(stderr, "# mark lines for %s.ppm\n", fname); 
+    /* or read from outside??? */
+    for (i = 0; i < lines->num; i++) { /* mark lines by 0x08 = blue */
+      dx = lines->x1[i] - lines->x0[i] + 1;
+      for (j = -1; j < dx+1; j++) {
+        x  = lines->x0[i] + j;
+        if (x<0 || x>=pp->x) continue;
+        for (y=lines->m1[i];y<=lines->m4[i];y++) {
+          /* box arround m2-m3 */
+          if (y>=lines->m2[i] && y<=lines->m3[i] && j>-1 && j<dx) continue;
+          yr  = y; /* y.rotated */
+          if (lines->dx) yr += lines->dy * x / (lines->dx);
+          if (yr<0 || yr>=pp->y) continue;
+          np = &(pp->p[x + (pp->x)*yr]);
+          if (*np<160) continue; /* do not touch dark pixels */
+          if ((*np&6)!=0) continue; /* only change white pixels */
+          put(pp, x, yr, 255, 6); /* UPN: 255 and 6 or */
+        } /* loop y */ 
+      } /* loop x */
+    } /* loop lines */
+  } /* opt & 4 */
 
-  if (job->cfg.verbose&1)
+  if ((job->cfg.verbose&33) || pp->x*pp->y <= 0)
     fprintf(stderr,"# writing %s[.png] xy= %d %d\n", fname, pp->x, pp->y);
-  writeppm(fname, pp);
+  if (pp->x*pp->y <= 0) exit(1); // generate error
+  writeppm(fname, pp, opt);
   return 0;
 }
