@@ -30,8 +30,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 void out_b(struct box *px, pix *b, int x0, int y0, int dx, int dy, int cs );
 void out_x(struct box *px);
 void out_x2(struct box *box1,struct box *box2);
-int output_list(pix * pp, char *lc);
-int write_img(char *fname, pix * ppo, pix * p, int opt);
+int output_list(job_t *job);
+int debug_img(char *fname, struct job_s *job, int opt);
 
 
 #endif
