@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000-2006  Joerg Schulenburg
+Copyright (C) 2000-2010  Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -171,7 +171,7 @@ bbg: FASTER now. It has 4 ifs less at least, 8 at most. */
         && ( (filt3[i][6]>>1) || c33[6]!=(1 & filt3[i][6]) )
         && ( (filt3[i][7]>>1) || c33[7]!=(1 & filt3[i][7]) )
         && ( (filt3[i][8]>>1) || c33[8]!=(1 & filt3[i][8]) ) ) {
-      return ((filt3[i][4])?JOB->cfg.cs:0);
+      return ((filt3[i][4])?OCR_JOB->cfg.cs:0);
     }
   return pixel_atp(p, x, y) & ~7;
 }
@@ -247,7 +247,7 @@ int pixel_filter_by_number(pix * p, int x, int y) {
   assert(val < NUM_TABLE_SIZE);
 
   if (num_table[val])
-      return (val & (1 << 4)) ? 0 : JOB->cfg.cs;
+      return (val & (1 << 4)) ? 0 : OCR_JOB->cfg.cs;
   else
     return pixel_atp(p, x, y) & ~7;
 }
@@ -423,13 +423,13 @@ int pixel_filter_by_tree(pix * p, int x, int y) {
 #endif
   if (tree[n] == 1) {
 #ifdef FILTER_STATISTICS
-    if (pixel_atp(p, x, y) < JOB->cfg.cs)
+    if (pixel_atp(p, x, y) < OCR_JOB->cfg.cs)
       filter_whitened++;
 #endif
-    return JOB->cfg.cs;
+    return OCR_JOB->cfg.cs;
   } else {
 #ifdef FILTER_STATISTICS
-    if (pixel_atp(p, x, y) >= JOB->cfg.cs)
+    if (pixel_atp(p, x, y) >= OCR_JOB->cfg.cs)
       filter_blackened++;
 #endif
     return 0;
@@ -485,13 +485,14 @@ void print_pixel_env(FILE * out, pix * p, int x, int y) {
  * Returns: pixel-color (without marks)
  */
 int getpixel(pix *p, int x, int y){
+  job_t *job=OCR_JOB; /* fixme */
   if ( x < 0 || y < 0 || x >= p->x || y >= p->y ) 
     return 255 & ~7;
 
   /* filter will be used only once later, when vectorization replaces pixel
    * processing 
    */
-  if (JOB->tmp.n_run > 0) { /* use the filters (correction of errors) */
+  if (job->tmp.n_run > 0) { /* use the filters (correction of errors) */
 #if FILTER_METHOD == FILTER_BY_NUMBER
     int pix = pixel_filter_by_number(p, x, y);
 #ifdef FILTER_CHECKED
