@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000-2010  Joerg Schulenburg
+Copyright (C) 2000-2018  Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -376,8 +376,15 @@ int readpgm( char *name, pix *p, int vvv){
     for (i=0;i<nx*ny;i++) {
      if (c2=='5') { if(bps!=(int)fread(buf,1,bps,f1)) {
         fprintf(stderr," ERROR reading at head+%d*%d\n", bps, i); break; } }
-     else for (j=0;j<3;j++) fread_num(buf+j*bps, bps, f1);
-     pic[i]=buf[bps-1]; /* store the most significant byte */
+     // P2 was buggy until 2018-09 read 3 values per pixel, fixed by F.O.
+     else fread_num(buf, bps, f1); // reading one number per pixel
+     // 2018-10 Endianess.NetPBM=MSB=H,L changed from L,H
+     // pic[i]=buf[bps-1]; /* store the most significant byte, L,H */
+     pic[i]=buf[0]; /* store the most significant byte, MSB=H,L 2018-10 */
+     // ToDo: no roundup if next bit is 1 (simpler code)
+     //if (i>451+203*3800 && i<455+203*3800)
+     //  fprintf(stderr,"\nDBG.rd %d,%d %3d %3d %3d - ",i%nx,i/nx,pic[i],
+     //    (buf[0]&255),(buf[1]&255));
     }
   // we want to normalize brightness to 0..255
   if (c2=='6' || c2=='3') {  // PPM-RAW/ASC
@@ -447,23 +454,24 @@ int writepgm(char *nam,pix *p){// P5 raw-pgm
   return 0;
 }
 
-/* adding colours, care about range */
+/* adding colours sr=red, sg=green, sb=blue, care about range */
 void addrgb(unsigned char rgb[3], int sr, int sg, int sb) {
  int add[3], i;
- /* add colour on dark pixels, subtract on white pixels */ 
+ /* add colour on dark pixels 0+.., subtract on white pixels 255-.. */ 
  add[0]=2*sr; add[1]=2*sg; add[2]=2*sb;
  if (((int)rgb[0])+((int)rgb[1])+((int)rgb[2])>=3*160)
     { add[0]=(-sg-sb); add[1]=(-sr-sb); add[2]=(-sr-sg); } // rgb/2?
  /* care about colour range */
- for (i=0;i<3;i++)
-   if (add[i]<0) rgb[i]-=((    rgb[i]<-add[i])?    rgb[i]:-add[i]);
-   else          rgb[i]+=((255-rgb[i]< add[i])?255-rgb[i]: add[i]);
+ for (i=0;i<3;i++) /* avoid overflow */
+   if (add[i]<0) rgb[i]=((rgb[i]+add[i]<0  )?  0:rgb[i]+add[i]);
+   else          rgb[i]=((rgb[i]+add[i]>255)?255:rgb[i]+add[i]);
 }
 /*
  * pgmtoppm or pnmtopng, use last 3 bits for farbcoding 
  * replaces old writebmp variant
+ *  2018-10 add $opt to control coloring
  */
-int writeppm(char *nam, pix *p){ /* P6 raw-ppm */
+int writeppm(char *nam, pix *p, int opt){ /* P6 raw-ppm */
   FILE *f1=NULL; int x,y,f1t=0; unsigned char rgb[3], gray, bits;
   char buf[128];
   if (strchr(nam,'|')) return -1;  /* no nasty code */
@@ -496,15 +504,17 @@ int writeppm(char *nam, pix *p){ /* P6 raw-ppm */
   for (y=0;y<p->y;y++)
   for (x=0;x<p->x;x++){
     gray=p->p[x+y*p->x];
-    bits=(gray&0x0F);  /* save marker bits */
+    bits=(gray&0x0F);  /* save 4 marker bits */
     /* replace used bits to get max. contrast, 160=0xA0 */
-    gray = ((gray<160) ? (gray&~0x0F)>>1 : 0xC3|(gray>>1) );
-    rgb[0] = rgb[1] = rgb[2] = gray;
-    if ((bits &  1)==1) { addrgb(rgb,0,0,8+8*((x+y)&1)); } /* dark blue */
-    if ((bits &  8)==8) { addrgb(rgb,0,0, 16); } /* blue (low priority) */
-    if ((bits &  6)==6) { addrgb(rgb,0,0, 32); } /* blue */
-    if ((bits &  6)==4) { addrgb(rgb,0,48,0); } /* green */
-    if ((bits &  6)==2) { addrgb(rgb,32,0,0); } /* red */
+    if (opt&7) gray = ((gray<160) ? (gray&~0x0F)>>1 : 0xC3|(gray>>1) );
+    rgb[0] = rgb[1] = rgb[2] = gray; /* add red,grn,blue */
+    if (opt&7) {
+      if ((bits &  1)==1) { addrgb(rgb,0,0,8+8*((x+y)&1)); } /* dark blue */
+      if ((bits &  8)==8) { addrgb(rgb,0,0, 16); } /* blue (low priority) */
+      if ((bits &  6)==6) { addrgb(rgb,0,0, 32); } /* blue */
+      if ((bits &  6)==4) { addrgb(rgb,0,48,0); } /* green */
+      if ((bits &  6)==2) { addrgb(rgb,32,0,0); } /* red */
+    }
     if ( 1!=(int)fwrite(rgb,3,1,f1) ) { E0("write"); y=p->y; break; }
   }
   if ( p->bpp==3 )
