@@ -27,7 +27,7 @@ int readpgm(char *name, pix *p, int vvv);
 /* write pgm-map to pnm-file */
 int writepgm(char *nam, pix *p);
 int writepbm(char *nam, pix *p);
-int writeppm(char *nam, pix *p); /* use lowest 3 bits for farbcoding */
+int writeppm(char *nam, pix *p, int opt); /* use lowest 3 bits for farbcoding */
 
 /* ----- count colors ------ create histogram ------- */
 void makehisto(pix p, unsigned col[256], int vvv);
