@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000-2010 Joerg Schulenburg
+Copyright (C) 2000-2017 Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -93,7 +93,7 @@ int remove_dust( job_t *job ){
       " obj=%d maxDust=%d mP= %3d mXY= %2d %2d",
       nC, job->cfg.dust_size, sP/nC, sX/nC, sY/nC);
     /* we assume that for random dust applies histo[i+1]<histo[i] */
-    for (i=1;i+3<HISTSIZE;i++){
+    for (i=1;i+3<HISTSIZE;i++){ /* dust size = frame_vol */
       if (vvv) fprintf(stderr,"\n#  dust size histogram %3d %5d",i,histo[i]);
       if (histo[i]>=nC) continue; /* v0.42 lot of pixels -> bg pattern < 3 */
       if (i>=job->cfg.dust_size) break;   /* maximum = mean size / 32 */
