@@ -1286,6 +1286,11 @@ const char *decode(wchar_t c, FORMAT type) {
         buf[3] = (char)0; /* terminate string */
         return buf;
       }
+      /* disable warnings on VS-2010 which seems to have 16bit wchar only */
+      /* echo "" | gcc -E - -dM | grep -i wchar # __WCHAR_MAX__ 2147483647 */
+      /*  if undefined, than UTF8 for more than 16bit will not work */
+      /*  do not now if __SIZEOF_WCHAR_T__ is defined on nonGCC compilers */
+#if __WCHAR_MAX__ > 0x10000  /* skip compile warnings on 16bit systems */
       if ( c <= (wchar_t)0x001FFFFF ) {  /* UTF8 == 21bit */
         buf[0] = (char)(0xf0|((c>>18) & 0x07)); /* 11110xxx */
         buf[1] = (char)(0x80|((c>>12) & 0x3f)); /* 10xxxxxx */
@@ -1313,6 +1318,7 @@ const char *decode(wchar_t c, FORMAT type) {
         buf[6] = (char)0; /* terminate string */
         return buf;
       }
+#endif  /* __WCHAR_MAX__ > 0x10000 */
       return (const char *)UNDEFINED;
   }
 }
